==> PEAR_2600_X64_LINUX_1.0.1.tar.gz: usr/local/openresty/nginx/lua/storage_common.lua <==
local router = require("router")
local cjson = require("cjson")

function storage_get_idle_drives()
	local file_handle = io.popen("lsblk -Jo name,path,mountpoint,vendor,model,serial,size,rota")
	local data = file_handle:read("*a");
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local dev_list = {}
	setmetatable(dev_list, cjson.empty_array_mt)
	for i = 1, n do
		local device = devices[i]
		local mountpoint = device["mountpoint"]
		if (mountpoint == ngx.null and device["children"] ~= nil) then
			local childdev = device["children"][1]
			mountpoint = childdev["mountpoint"]
			if (mountpoint == ngx.null and childdev["children"] ~= nil) then
				childdev = childdev["children"][1]
				mountpoint = childdev["mountpoint"]
			end
		end

		if (mountpoint == ngx.null) then
			device["children"] = nil
			device["mountpoint"] = nil
			device["rota"] = device["rota"] and "HDD" or "SSD"
			table.insert(dev_list, device)
		end
	end

	local temp = {}
	temp["drives"] = dev_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

function storage_get_drive()
	local path = ngx.var.arg_path
	if (path == nil) then
		router.process_error(5, "server get url args failed!")
		return -1
	end

	local file_handle = io.popen("lsblk -Jo name,path,vendor,model,serial,size,rota,type")
	local data = file_handle:read("*a");
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local drive = {}
	for i = 1, n do
		local device = devices[i]
		if (device["children"] ~= nil) then
			local m = table.getn(device["children"])
			for j = 1, m do
				local childdev = device["children"][j]
				if (childdev["path"] == path) then
					device["children"] = nil
					device["name"] = path
					device["rota"] = device["rota"] and "HDD" or "SSD"
					local resp = cjson.encode(device)
					ngx.say(resp)
					return 0
				end
			end
		end
	end

	router.process_error(4, "server get device info failed!")
	return -1
end

function storage_get_pool()
	local path = ngx.var.arg_path
	if (path == nil) then
		router.process_error(5, "server get url args failed!")
		return -1
	end

	local file_handle = io.popen("lsblk -Jo name,path,vendor,model,serial,size,rota,type")
	local data = file_handle:read("*a");
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local drives_list = {}
	setmetatable(drives_list, cjson.empty_array_mt)

	for i = 1, n do
		local device = devices[i]
		if (device["children"] ~= nil) then
			local m = table.getn(device["children"])
			for j = 1, m do
				local childdev = device["children"][j]
				if (childdev["children"] ~= nil) then
					childdev = childdev["children"][1]
				end
				if (childdev["path"] == path) then
					device["children"] = nil
					device["rota"] = device["rota"] and "HDD" or "SSD"
					table.insert(drives_list, device)
				end
			end
		end
	end

	local temp = {}
	temp["name"] = path
	temp["drives"] = drives_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

function storage_get_volumes()
	local file_handle = io.popen("lsblk -Jo path,mountpoint,size,fsused,fsuse%,fstype,type")
	local data = file_handle:read("*a");
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local volumes_list = {}
	local mount_list = {}
	setmetatable(volumes_list, cjson.empty_array_mt)

	for i = 1, n do
		local device = devices[i]
		local mountpoint = device["mountpoint"]
		if (mountpoint ~= ngx.null) then
			local start,_ = string.find(mountpoint, "/data(%d+)")
			if (start == 1 and mount_list[mountpoint] == nil) then
				mount_list[mountpoint] = device
				device["name"] = mountpoint
				device["mountpoint"] = nil
				table.insert(volumes_list, device)
			end
		end
	end

	local temp = {}
	temp["volumes"] = volumes_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

function storage_create_pool()
	local body_data = ngx.req.get_body_data()
	local file = io.open("/etc/router/config/storage_create_pool.json", "w");
	if (file == nil) then
		router.process_error(2, "write_config: open file failed!")
		return -1
	end
	file:write(body_data)
	file:close()

	local os_ret = os.execute("/etc/router/scripts/storage_create_pool.sh")
	if (os_ret == nil) then
		router.process_error(3, "server operation failed!")
		return -1
	end

	local req = cjson.decode(body_data)
	local op_cmd = "lsblk -Jo path,type "..req["drives"][1]
	local file_handle = io.popen(op_cmd)
	local data = file_handle:read("*all")
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	for i = 1, n do
		local device = devices[i]
		if (device["type"] == req["raid_level"]) then
			local resp = cjson.encode(device)
			ngx.say(resp)
			return 0
		end
	end

	router.process_error(4, "server get device info failed!")
	return -1
end

function storage_create_volume()
	router.write_config("/etc/router/config/storage_create_volume.json", "/etc/router/scripts/storage_create_volume.sh")
end

function storage_delete_volume()
	router.write_config("/etc/router/config/storage_delete_volume.json", "/etc/router/scripts/storage_delete_volume.sh")
end

if (ngx.var.uri == '/storage/drives/idle') then
	if (ngx.var.request_method == 'GET') then
		storage_get_idle_drives()
	end
elseif (ngx.var.uri == '/storage/drive') then
	if (ngx.var.request_method == 'GET') then
		storage_get_drive()
	end
elseif (ngx.var.uri == '/storage/pool') then
	if (ngx.var.request_method == 'GET') then
		storage_get_pool()
	elseif (ngx.var.request_method == 'POST') then
		storage_create_pool()
	end
elseif (ngx.var.uri == '/storage/volumes') then
	if (ngx.var.request_method == 'GET') then
		storage_get_volumes()
	end
elseif (ngx.var.uri == '/storage/volume') then
	if (ngx.var.request_method == 'POST') then
		storage_create_volume()
	elseif (ngx.var.request_method == 'DELETE') then
		storage_delete_volume()
	end
else
	router.process_error(1, "invalid uri: "..ngx.var.uri)
end

==> PEAR_2600_X64_LINUX_1.2.0.tar.gz: usr/local/openresty/nginx/lua/storage_common.lua <==
local router = require("router")
local cjson = require("cjson")

function storage_get_device_serial(device)
	local serial = device["serial"]
	local os_ret = os.execute("hdparm -I "..device["path"])
	if (os_ret == true) then
		local cmd = "hdparm -I "..device["path"].." | grep 'Serial Number' | awk -F: '{print $2}' | tr -d ' '"
		local pf = io.popen(cmd)
		serial = pf:read()
		pf:close()
	end
	return serial
end

function storage_get_idle_drives()
	local file_handle = io.popen("lsblk -Jo name,path,mountpoint,vendor,model,serial,size,fsused,fsuse%,rota")
	local data = file_handle:read("*a")
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local dev_list = {}
	setmetatable(dev_list, cjson.empty_array_mt)
	for i = 1, n do
		local device = devices[i]
		local mountpoint = device["mountpoint"]
		if (mountpoint == ngx.null and device["children"] ~= nil) then
			local childdev = device["children"][1]
			mountpoint = childdev["mountpoint"]
			if (mountpoint == ngx.null and childdev["children"] ~= nil) then
				childdev = childdev["children"][1]
				mountpoint = childdev["mountpoint"]
			end
		end

		if (mountpoint == ngx.null) then
			-- device["children"] = nil
			device["mountpoint"] = nil
			device["rota"] = device["rota"] and "HDD" or "SSD"
			device["serial"] = storage_get_device_serial(device)
			table.insert(dev_list, device)
		end
	end

	local temp = {}
	temp["drives"] = dev_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

function storage_get_drive()
	local path = ngx.var.arg_path
	if (path == nil) then
		router.process_error(5, "server get url args failed!")
		return -1
	end

	local file_handle = io.popen("lsblk -Jo name,path,vendor,model,serial,size,fsused,fsuse%,rota,type")
	local data = file_handle:read("*a")
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local drive = {}
	for i = 1, n do
		local device = devices[i]
		if (device["children"] ~= nil) then
			local m = table.getn(device["children"])
			for j = 1, m do
				local childdev = device["children"][j]
				if (childdev["path"] == path) then
					-- device["children"] = nil
					device["name"] = path
					device["rota"] = device["rota"] and "HDD" or "SSD"
					device["serial"] = storage_get_device_serial(device)
					local resp = cjson.encode(device)
					ngx.say(resp)
					return 0
				end
			end
		end
	end

	router.process_error(4, "server get device info failed!")
	return -1
end

function storage_get_pool()
	local path = ngx.var.arg_path
	if (path == nil) then
		router.process_error(5, "server get url args failed!")
		return -1
	end

	local file_handle = io.popen("lsblk -Jo name,path,vendor,model,serial,size,fsused,fsuse%,rota,type")
	local data = file_handle:read("*a")
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local drives_list = {}
	setmetatable(drives_list, cjson.empty_array_mt)

	for i = 1, n do
		local device = devices[i]
		if (device["children"] ~= nil) then
			local m = table.getn(device["children"])
			for j = 1, m do
				local childdev = device["children"][j]
				if (childdev["children"] ~= nil) then
					childdev = childdev["children"][1]
				end
				if (childdev["path"] == path) then
					-- device["children"] = nil
					device["rota"] = device["rota"] and "HDD" or "SSD"
					device["serial"] = storage_get_device_serial(device)
					table.insert(drives_list, device)
				end
			end
		end
	end

	local temp = {}
	temp["name"] = path
	temp["drives"] = drives_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

function storage_get_volumes()
	local file_handle = io.popen("lsblk -Jo path,mountpoint,size,fsused,fsuse%,fstype,type")
	local data = file_handle:read("*a")
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	local volumes_list = {}
	local mount_list = {}
	setmetatable(volumes_list, cjson.empty_array_mt)

	for i = 1, n do
		local device = devices[i]
		local mountpoint = device["mountpoint"]
		if (mountpoint ~= ngx.null) then
			local start,_ = string.find(mountpoint, "/data(%d+)")
			if (start == 1 and mount_list[mountpoint] == nil) then
				mount_list[mountpoint] = device
				device["name"] = mountpoint
				device["mountpoint"] = nil
				table.insert(volumes_list, device)
			end
		end
	end

	local temp = {}
	temp["volumes"] = volumes_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

function storage_create_pool()
	local body_data = ngx.req.get_body_data()
	local file = io.open("/etc/router/config/storage_create_pool.json", "w");
	if (file == nil) then
		router.process_error(2, "write_config: open file failed!")
		return -1
	end
	file:write(body_data)
	file:close()

	local os_ret = os.execute("/etc/router/scripts/storage_create_pool.sh")
	if (os_ret == nil) then
		router.process_error(3, "server operation failed!")
		return -1
	end

	local req = cjson.decode(body_data)
	local op_cmd = "lsblk -Jo path,type "..req["drives"][1]
	local file_handle = io.popen(op_cmd)
	local data = file_handle:read("*all")
	local table_data = cjson.decode(data)
	local devices = table_data["blockdevices"]
	local n = table.getn(devices)
	for i = 1, n do
		local device = devices[i]
		if (device["type"] == req["raid_level"]) then
			local resp = cjson.encode(device)
			ngx.say(resp)
			return 0
		end
	end

	router.process_error(4, "server get device info failed!")
	return -1
end

function storage_create_volume()
	router.write_config("/etc/router/config/storage_create_volume.json", "/etc/router/scripts/storage_create_volume.sh")
end

function storage_delete_volume()
	router.write_config("/etc/router/config/storage_delete_volume.json", "/etc/router/scripts/storage_delete_volume.sh")
end

if (ngx.var.uri == '/storage/drives/idle') then
	if (ngx.var.request_method == 'GET') then
		storage_get_idle_drives()
	end
elseif (ngx.var.uri == '/storage/drive') then
	if (ngx.var.request_method == 'GET') then
		storage_get_drive()
	end
elseif (ngx.var.uri == '/storage/pool') then
	if (ngx.var.request_method == 'GET') then
		storage_get_pool()
	elseif (ngx.var.request_method == 'POST') then
		storage_create_pool()
	end
elseif (ngx.var.uri == '/storage/volumes') then
	if (ngx.var.request_method == 'GET') then
		storage_get_volumes()
	end
elseif (ngx.var.uri == '/storage/volume') then
	if (ngx.var.request_method == 'POST') then
		storage_create_volume()
	elseif (ngx.var.request_method == 'DELETE') then
		storage_delete_volume()
	end
else
	router.process_error(1, "invalid uri: "..ngx.var.uri)
end
